import os
import pandas as pd
from subtotals import groupby
from pandas.testing import assert_frame_equal as afe

folder = os.path.dirname(os.path.abspath(__file__))
sales_file = os.path.join(folder, 'sales.csv')
sales_data: pd.DataFrame = pd.read_csv(sales_file)


def test_subtotal():
    agg = {'sales': 'sum', 'growth': 'min'}
    cols = ['country', 'city', 'product', 'sales', 'growth']
    actual = groupby(sales_data, ['country', 'city', 'product'], agg, margins_name='All')

    l0 = sales_data.groupby([0] * len(sales_data), sort=False).agg(agg)
    l0['country'] = l0['city'] = l0['product'] = 'All'
    afe(actual[actual['level'] == 0][cols].reset_index(drop=True), l0[cols])

    l1 = sales_data.groupby('country', sort=False).agg(agg).reset_index()
    l1['city'] = l1['product'] = 'All'
    afe(actual[actual['level'] == 1][cols].reset_index(drop=True), l1[cols])

    l2 = sales_data.groupby(['country', 'city'], sort=False).agg(agg).reset_index()
    l2['product'] = 'All'
    afe(actual[actual['level'] == 2][cols].reset_index(drop=True), l2[cols])

    l3 = sales_data.groupby(['country', 'city', 'product'], sort=False).agg(agg).reset_index()
    afe(actual[actual['level'] == 3][cols].reset_index(drop=True), l3[cols])



'''Generate subtotals for Pandas DataFrames.'''

import pandas as pd
from typing import Union, List, Dict, Callable

__version__ = '0.1.1'


def groupby(
    data: pd.DataFrame,
    by: List[str],
    agg: Union[Callable, str, List[Union[str, Callable]], Dict[str, Union[str, Callable]]],
    margins_name: str = 'Total',
) -> pd.DataFrame:
    '''Group a Pandas DataFrame with subtotals.

    Parameters:

        data: The data to subtotal
        by: Levels by which the data must be grouped
        agg: How to aggregated columns. See Pandas
            [GroupBy.agg](https://pandas.pydata.org/docs/reference/api/pandas.core.groupby.DataFrameGroupBy.agg.html)
        margins_name: Name of the row that contains the subtotals

    Returns:
        Subtotaled DataFrame
    '''
    grand_total = data.groupby(pd.Series(0, data.index), sort=False).agg(agg)
    grand_total['level'] = 0
    frames = [grand_total]
    for level in range(1, 1 + len(by)):
        frame = data.groupby(by[:level], sort=False, as_index=False).agg(agg)
        frame['level'] = level
        frames.append(frame)
    result = pd.concat(frames, ignore_index=True)
    if margins_name is not None:
        result[by] = result[by].astype(object).fillna(margins_name)
    return result
